# Generated by Django 3.1.13 on 2022-01-12 05:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("nautobot_ssot", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="sync",
            name="diff_memory_final",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="diff_memory_peak",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="diff_time",
            field=models.DurationField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="source_load_memory_final",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="source_load_memory_peak",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="source_load_time",
            field=models.DurationField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="sync_memory_final",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="sync_memory_peak",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="sync_time",
            field=models.DurationField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="target_load_memory_final",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="target_load_memory_peak",
            field=models.PositiveBigIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="sync",
            name="target_load_time",
            field=models.DurationField(blank=True, null=True),
        ),
    ]
